#ifndef ___CFLOATEX_H_ONCE
#define ___CFLOATEX_H_ONCE

#include "floatex.h"

class CFloatEX {
	FLOATEX 		val;
	char			private_buf[32];
public:
	// init.cpp
	CFloatEX(const CFloatEX &);
	CFloatEX(const long);
	CFloatEX(const unsigned long);
	CFloatEX(const double);
	CFloatEX(const INT64);
	CFloatEX(const UINT64);
	CFloatEX(const char *);
	CFloatEX();
	~CFloatEX();

	// conv.cpp
	CFloatEX		operator=(const long);
	CFloatEX		operator=(const unsigned long);
	CFloatEX		operator=(const double);
	CFloatEX		operator=(const INT64);
	CFloatEX		operator=(const UINT64);
	CFloatEX		operator=(const char *);
	long			to_long(void);
	unsigned long	to_ulong(void);
	double			to_double(void);
	INT64			to_int64(void);
	UINT64			to_uint64(void);
	char			*to_str(const char *pbuffer = 0);
	char			*to_str(const char *,int,const char *);
	CFloatEX		str_to(const char *pstr,char **ppstop = 0);

	// calc.cpp

	CFloatEX		operator+(void);
	CFloatEX		operator-(void);
	CFloatEX		operator++(void);
	CFloatEX		operator--(void);
	CFloatEX		operator+(const CFloatEX &);
	CFloatEX		operator-(const CFloatEX &);
	CFloatEX		operator*(const CFloatEX &);
	CFloatEX		operator/(const CFloatEX &);
	CFloatEX		operator%(const CFloatEX &);
	CFloatEX		operator+(const double);
	CFloatEX		operator-(const double);
	CFloatEX		operator*(const double);
	CFloatEX		operator/(const double);
	CFloatEX		operator%(const double);
	CFloatEX		operator+=(const CFloatEX &);
	CFloatEX		operator-=(const CFloatEX &);
	CFloatEX		operator*=(const CFloatEX &);
	CFloatEX		operator/=(const CFloatEX &);
	CFloatEX		operator%=(const CFloatEX &);
	CFloatEX		operator+=(const double);
	CFloatEX		operator-=(const double);
	CFloatEX		operator*=(const double);
	CFloatEX		operator/=(const double);
	CFloatEX		operator%=(const double);

	// comp.cpp
	int 			operator==(const CFloatEX &);
	int 			operator!=(const CFloatEX &);
	int 			operator<(const CFloatEX &);
	int 			operator>(const CFloatEX &);
	int 			operator<=(const CFloatEX &);
	int 			operator>=(const CFloatEX &);
	int 			operator==(const double);
	int 			operator!=(const double);
	int 			operator<(const double);
	int 			operator>(const double);
	int 			operator<=(const double);
	int 			operator>=(const double);
	int 			operator!(void);

	// float.cpp
	CFloatEX		abs(void);
	CFloatEX		nabs(void);
	CFloatEX		chsign(void);
	CFloatEX		copysign(const CFloatEX &); // Rs[lw
	CFloatEX		integer(void);
	int 			sgn(void);
	CFloatEX		nextafter(const CFloatEX &); // y w

	// ʂIuWFNgɊi[֐
	CFloatEX		fval(int,UINT64,int,int);
	CFloatEX		qnan(UINT64 mant = 0ui64);
	CFloatEX		snan(UINT64 mant = 0ui64);

	// math.cpp
	CFloatEX		sin(void);
	CFloatEX		cos(void);
	CFloatEX		tan(void);
	CFloatEX		asin(void);
	CFloatEX		acos(void);
	CFloatEX		atan(void);

	CFloatEX		sinh(void);
	CFloatEX		cosh(void);
	CFloatEX		tanh(void);
	CFloatEX		asinh(void);
	CFloatEX		acosh(void);
	CFloatEX		atanh(void);

	CFloatEX		sqrt(void);
	CFloatEX		cbrt(void);
	CFloatEX		exp(void);
	CFloatEX		ln(void);
	CFloatEX		log(void);
	CFloatEX		log2(void);
	CFloatEX		logxy(const CFloatEX &); // ^w
	CFloatEX		pow(const CFloatEX &); // ww
	CFloatEX		pow2(void);
	CFloatEX		pow10(void);

	CFloatEX		hypot(const CFloatEX &); // x ܂ y w

	CFloatEX		floor(void);
	CFloatEX		ceil(void);

	CFloatEX		jn(int);
	CFloatEX		yn(int);
	CFloatEX		gamma(void);

	CFloatEX		fact(void);
	CFloatEX		dist_normal_u(void);
	CFloatEX		dist_normal_d(void);
	CFloatEX		dist_t_u(int);
	CFloatEX		dist_t_d(int);
	CFloatEX		dist_chi_u(int);
	CFloatEX		dist_chi_d(int);
	CFloatEX		dist_f_u(int,int);
	CFloatEX		dist_f_d(int,int);

	// ʂIuWFNgɊi[֐

	CFloatEX		atan(const CFloatEX &,const CFloatEX &);
	CFloatEX		logxy(const CFloatEX &,const CFloatEX &);
	CFloatEX		pow(const CFloatEX &,const CFloatEX &);
	CFloatEX		hypot(const CFloatEX &,const CFloatEX &);
	CFloatEX		distance(
						const CFloatEX &,const CFloatEX &,
						const CFloatEX &,const CFloatEX &);
	CFloatEX		lcm(const CFloatEX &,const CFloatEX &);
	CFloatEX		gcm(const CFloatEX &,const CFloatEX &);
	CFloatEX		permutation(const CFloatEX &,const CFloatEX &);
	CFloatEX		combination(const CFloatEX &,const CFloatEX &);
};

#endif
